#!/usr/bin/python3

# sale_widget.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" sale widget """

import sys
import os
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir1)

if __name__ == '__main__':
    from cashbox.app import App, MinWindow
    from cashbox.article import Article, Sale

try:
    from cashbox.pick_widget import PickWidget
    from cashbox.drop_down_widget import DropDownWidget
    from cashbox.utils import create_action
    from cashbox.dialog_widget import DialogWidget
    from cashbox.locale_utils import _, f
except ImportError as exc:
    print('Error: decision modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Adw', '1')
    gi.require_version('Gtk', '4.0')
    from gi.repository import Gtk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f'{dir1}/sale_widget.ui')
class SaleWidget(Gtk.Box):
    """ SaleWidget """
    __gtype_name__ = 'SaleWidget'
    sale_widget_menu_box = Gtk.Template.Child()
    sale_widget_box = Gtk.Template.Child()

    def do_pick_item(self, dropdown, _b):
        """ x """
        if dropdown.get_selected() > 0:
            article = dropdown.get_selected_item()
            dropdown.set_selected(0)
            article.count = 1

    def __init__(self, sale, win, **kwargs):
        super().__init__(**kwargs)

        self.sale = sale
        self.win = win

        drop_down_widget = DropDownWidget(sale.drop_unpicked)
        drop_down = drop_down_widget.get_drop_down()

        drop_down.connect("notify::selected-item", self.do_pick_item)
        self.sale_widget_menu_box.append(drop_down_widget)

        pick = PickWidget(sale)
        self.sale_widget_box.append(pick)

        create_action(win, "help_sale", self.on_help_sale)

    def on_help_sale(self, _action, _param):
        """ x """
        d = DialogWidget()
        win = self
        d.help_dialog(win, _("Sale Help"), f(_("""\
To sell {d.as_}, they can be selected from a drop down menu. They will then \
be shown in the order selected in the {d.S} widget, where the {d.c} to be \
sold can be adjusted.

The {d.as_} in the drop down menu always have the same order as in the \
{d.Pl}. \
If some {d.as_} are sold more often than others, they may be moved to the \
beginning in the {d.Pl}, to be able to find and select them faster in future.
""")))


if __name__ == '__main__':

    class SaleWindow(MinWindow):
        """ x """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            sale = Sale()

            article = [("Banana", 110, 0), ("Apple", 200, 0),
                       ("Strawberry", 250, 0), ("Pear", 335, 4),
                       ("Watermelon", 100, 5), ("Blueberry", 200, 6)]

            for a in article:
                sale.main_list.append(Article(a[0], a[1], a[2]))

            box = Gtk.Box(spacing=12, hexpand=True, vexpand=True)
            box.props.margin_start = 12
            box.props.margin_end = 12
            box.props.margin_top = 6
            box.props.margin_bottom = 6
            self.set_content(box)

            sale_widget = SaleWidget(sale, win=self)
            box.append(sale_widget)

    class MyApp(App):
        """ x """

        def on_activate(self, app):
            win = SaleWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# receipt_widget.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" receipt widget """

import sys
import os
import pathlib
from datetime import datetime
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir1)

if __name__ == '__main__':
    from cashbox.app import App, MinWindow

try:
    from cashbox.utils import eprint, create_action
    from cashbox.read_appargs import appargs
    from cashbox.article import Article, Sale, cent2str, str2cent
    from cashbox.cshbx import Cshbx
    from cashbox.dialog_widget import DialogWidget
    from cashbox.locale_utils import _, f
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Gtk, Gio, GObject
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f'{dir1}/receipt_widget_row.ui')
class ReceiptWidgetRow(Gtk.Box):
    """ Receipt Widget """
    __gtype_name__ = 'ReceiptWidgetRow'
    action_row = Gtk.Template.Child()
    count_label = Gtk.Template.Child()
    price_label = Gtk.Template.Child()
    sum_label = Gtk.Template.Child()


@Gtk.Template(filename=f'{dir1}/receipt_widget_dialog.ui')
class ReceiptWidgetDialog(Gtk.Box):
    """ ReceiptWidgetDialog """
    __gtype_name__ = 'ReceiptWidgetDialog'
    statistic_dialog = Gtk.Template.Child()  # Adw.Dialog
    session = Gtk.Template.Child()
    sales = Gtk.Template.Child()
    revenue = Gtk.Template.Child()

    @Gtk.Template.Callback()
    def on_statistic_dialog_end(self, _button):
        """ x """
        self.statistic_dialog.close()


@Gtk.Template(filename=f'{dir1}/receipt_widget.ui')
class ReceiptWidget(Gtk.Box):
    """ x """
    __gtype_name__ = 'ReceiptWidget'
    list_view = Gtk.Template.Child()
    money_sum = Gtk.Template.Child()
    money_in = Gtk.Template.Child()
    money_out = Gtk.Template.Child()
    file_dialog = Gtk.Template.Child()  # Gtk.FileDialog
    ok_button = Gtk.Template.Child()

    def __init__(self, sale, win, **kwargs):
        super().__init__(**kwargs)

        # win
        self.win = win
        self.cshbx = Cshbx()

        # prepare on_money_in
        self.money_in_buffer = self.money_in.get_buffer()
        self.money_in_buffer.connect('notify', self.on_money_in)
        self.money_sum_buffer = self.money_sum.get_buffer()
        self.money_out_buffer = self.money_out.get_buffer()

        # label_sizegroup
        self.sizegroups = {"name": Gtk.SizeGroup(), "count": Gtk.SizeGroup(),
                           "price": Gtk.SizeGroup(), "sum": Gtk.SizeGroup()}
        self.sizegroups["name"].set_mode(Gtk.SizeGroupMode.HORIZONTAL)
        self.sizegroups["count"].set_mode(Gtk.SizeGroupMode.HORIZONTAL)
        self.sizegroups["price"].set_mode(Gtk.SizeGroupMode.HORIZONTAL)
        self.sizegroups["sum"].set_mode(Gtk.SizeGroupMode.HORIZONTAL)

        # SingleSelection
        selection = Gtk.NoSelection()
        self.list_view.set_model(selection)

        # sale
        self.sale = sale
        selection.set_model(self.sale.picked)

        # factory
        factory = Gtk.SignalListItemFactory()
        self.list_view.set_factory(factory)
        factory.connect("setup", self.on_factory_setup)
        factory.connect("bind", self.on_factory_bind)

        # action
        create_action(win, "chng_session_dir", self.on_chng_session_dir)
        create_action(win, "show_statistic", self.on_show_statistic)
        create_action(win, "help_receipt", self.on_help_receipt)

    def on_help_receipt(self, _action, _param):
        """ x """
        d = DialogWidget()
        win = self
        d.help_dialog(win, _("Receipt Help"), f(_("""\
In the {d.R} window the {d.p} to pay is shown. The paid amount can be entered \
and the return will be shown. With 'Ok' the sale will be saved and the \
selected {d.a} {d.cs} will be resetted for the next customer.""")))

    def on_money_in(self, _a, _b):
        """ x """
        money_sum = str2cent(self.money_sum_buffer.get_text())
        money_in_text = self.money_in_buffer.get_text()
        money_out_text = ""
        money_in = str2cent(money_in_text, True)
        if money_in and money_sum:
            if money_in > money_sum:
                money_out_text = cent2str(money_in - money_sum)
        self.money_out_buffer.set_text(money_out_text, len(money_out_text))

    @Gtk.Template.Callback()
    def on_map_sum(self, _widget):
        """ x """
        paysum = 0
        for article in self.sale.picked:
            paysum += article.price * article.count
            print(f"sum=<{sum}>")
        if paysum > 0:
            self.ok_button.set_sensitive(True)
        else:
            self.ok_button.set_sensitive(False)
        s = cent2str(paysum)
        print(f"on_map_sum will set <{s}>")
        self.money_sum_buffer.set_text(s, len(s))

    @Gtk.Template.Callback()
    def on_save_receipt(self, button):
        """ x """
        eprint(f"on_savereceipt: button=<{button}>")
        session_dir = os.path.join(appargs.user_app_dir, appargs.session)
        pathlib.Path(session_dir).mkdir(parents=True, exist_ok=True)
        save_time = datetime.now().strftime('%H-%M-%S')
        sale_file = os.path.join(session_dir, save_time + ".cshbx")
        eprint(f"sale_file=<{sale_file}>")
        eprint(f"txt=<{self.sale.text()}>")
        with open(sale_file, 'w', encoding="utf-8") as file:
            file.write(self.sale.text())
        self.sale.count_zero()
        self.money_in_buffer.delete_text(0, -1)
        self.money_sum_buffer.delete_text(0, -1)
        self.ok_button.set_sensitive(False)

    def on_factory_setup(self, _fact, item):
        """ x """
        receipt_row = ReceiptWidgetRow()
        item.set_child(receipt_row)
        receipt_row.connect('map', self.on_map_row, item)

    def on_factory_bind(self, _fact, item):
        """ x """
        article = item.get_item()
        receipt_row = item.get_child()
        self.sizegroups["price"].add_widget(receipt_row.price_label)
        self.sizegroups["count"].add_widget(receipt_row.count_label)
        self.sizegroups["sum"].add_widget(receipt_row.sum_label)

        article.bind_property("name",
                              receipt_row.action_row, "title",
                              GObject.BindingFlags.SYNC_CREATE)

    def on_map_row(self, receipt_row, item):
        """ x """
        article = item.get_item()
        receipt_row.count_label.set_label(f"{article.count}")
        receipt_row.price_label.set_label(cent2str(article.price))
        receipt_row.sum_label.set_label(cent2str(article.price *
                                        article.count))

    def on_chng_session_dir(self, _action, _param):
        """ x """
        session_dir = os.path.join(appargs.user_app_dir, appargs.session)
        pathlib.Path(session_dir).mkdir(parents=True, exist_ok=True)
        self.file_dialog.set_initial_folder(Gio.File.new_for_path(session_dir))
        self.file_dialog.select_folder(None, None,
                                       self.on_chng_session_dir_finish)

    def on_chng_session_dir_finish(self, dialog, task):
        """ x """
        file = dialog.select_folder_finish(task)
        if file:
            path = file.get_path()
            appargs.session = os.path.basename(path)

    def on_show_statistic(self, _action, _param):
        """ x """
        session_dir = os.path.join(appargs.user_app_dir, appargs.session)
        names = [name for name in os.listdir(session_dir) if
                 os.path.isfile(os.path.join(session_dir, name))]

        revenue = 0
        for name in names:
            with open(os.path.join(session_dir, name), 'r',
                      encoding="utf-8") as file:
                for line in file:
                    revenue += self.cshbx.get_cent_price(line)

        receipt_widget_dialog = ReceiptWidgetDialog()
        receipt_widget_dialog.session.set_label(appargs.session)
        receipt_widget_dialog.sales.set_label(f"{len(names)}")
        receipt_widget_dialog.revenue.set_label(f"{revenue/appargs.cents}")

        statistic_dialog = receipt_widget_dialog.statistic_dialog
        statistic_dialog.present(self.win)


if __name__ == '__main__':

    class ReceiptWindow(MinWindow):
        """ x """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            self.sale = Sale()

            self.box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
            self.set_content(self.box)

            list_sale_button = Gtk.Button(label="list")
            self.box.append(list_sale_button)
            list_sale_button.connect('clicked', self.list_sale_clicked)

            self.pick_widget = ReceiptWidget(self.sale, win=self)
            self.box.append(self.pick_widget)

            quit_button = Gtk.Button(label="quit")
            self.box.append(quit_button)
            quit_button.connect('clicked', self.quit_clicked)

            article = [("Banana", 110, 1), ("Apple", 200, 2),
                       ("Strawberry", 250, 3), ("Pear", 335, 4),
                       ("Watermelon", 100, 5), ("Blueberry", 200, 6)]

            for a in article:
                self.sale.main_list.append(Article(a[0], a[1], a[2]))

        def list_sale_clicked(self, _list_button):
            """ x """
            print(f"list_sale_clicked: sale=<{self.sale}>")

        def quit_clicked(self, _list_button):
            """ x """
            self.close()

    class MyApp(App):
        """ x """

        def on_activate(self, app):
            win = ReceiptWindow(application=app)
            win.present()

    # appargs.read_appargs()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# cshbx.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" provide Class cshbx """

import sys
import os
import re

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir1)

try:
    from cashbox.read_appargs import appargs
    from cashbox.app import App
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)


class Cshbx():
    """
    a text line with an article needs an article name and and a price
    optionally it can contain a count
    """

    def __init__(self):
        self.re_price = None
        self.re_name = None
        self.re_count = None
        self.re_some = None
        self.compile_re()

    def compile_re(self):
        """ compile regular expressions """
        re_price = (r'(\$\s*|\€\s*|)(\d+)(,|\.)(\d\d)'
                    r'(\s*\$|\s*€|\s*Dollar|\s*Euro|)')
        re_name = r'(\s*)(.*\S)(\s+)'
        re_count = r'(?:(\s+)(\d+)){0,1}(\s*)$'
        self.re_sale = re.compile(r'^'+re_name+re_price+re_count+r'$')

        self.re_price = re.compile(re_price)
        self.re_name = re.compile(r'^' + re_name + r'$')
        self.re_count = re.compile(r'^' + re_count + r'$')
        self.re_some = re.compile(r'^(\s*)(.*\S)(\s*)$')

    def get_cent_price(self, line):
        """ x """
        res = 0
        match = self.re_sale.match(line)
        if match:
            groups = match.groups()
            if groups[9]:
                res += ((int(groups[4])*appargs.cents + int(groups[6])) *
                        int(groups[9]))
        return res


if __name__ == '__main__':

    App().run(sys.argv)
    cshbx = Cshbx()

    assert cshbx.re_price.match("1,20").groups() == ('', '1', ',', '20', '')
    assert (cshbx.re_price.match("1.20 Dollar").groups() ==
            ('', '1', '.', '20', ' Dollar'))
    assert (list(cshbx.re_price.finditer("Bier 0,25 Liter 1,10€"))[-1].groups()
            == ('', '1', ',', '10', '€'))
    assert (list(cshbx.re_price.finditer("0.25 Dollar"))[-1].groups() ==
            ('', '0', '.', '25', ' Dollar'))
    assert (list(cshbx.re_price.finditer("$0.25 Dollar"))[-1].groups() ==
            ('$', '0', '.', '25', ' Dollar'))

    assert cshbx.get_cent_price("garbage") == 0
    assert cshbx.get_cent_price("a 1,20 5") == 600

    assert cshbx.re_price.match("12,34").groups() == ('', '12', ',', '34', '')
    assert (cshbx.re_price.match("12,34 €").groups() ==
            ('', '12', ',', '34', ' €'))
    assert (cshbx.re_price.match("12.34$").groups() ==
            ('', '12', '.', '34', '$'))
    assert cshbx.re_price.match("12.3") is None

    print("all asserts have been ok")



#!/usr/bin/python3

# utils.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" utils """

import sys
import gi

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Gio
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


def widget_info(widget, level=1):
    """ widget_info

    >>> from gi.repository import Gtk
    >>> box=Gtk.Box()
    >>> label=Gtk.Label(label="mylabel")
    >>> box.append(label)
    >>> print(widget_info(box))
    - <GtkBox>
    -- <GtkLabel> label=<mylabel>

    """

    ret = ""
    if widget:
        indent = "-"*level
        txt = widget.get_name()
        ret = f"{ret}{indent} <{txt}>"
        blp_id = widget.get_buildable_id()  # id defined in *.blp or *.ui
        if blp_id:
            ret = f"{ret} id=<{blp_id}>"
        if txt == "GtkLabel":
            label = widget.get_label()
            if label:
                ret = f"{ret} label=<{label}>"
    if widget:
        widget = widget.get_first_child()
        while widget:
            ret = f"{ret}\n{widget_info(widget, level+1)}"
            widget = widget.get_next_sibling()
    return ret


def reduce_window_size(window, x=350, y=600):
    """ x """
    window.props.default_width = x
    window.props.default_height = y


if __name__ == '__main__':
    import doctest
    doctest.testmod()


def eprint(*args, **kwargs):
    """ print on stderr """
    print(*args, file=sys.stderr, **kwargs)


def err(msg, retcode=1):
    """ print error and exit """
    eprint(f"Error: {msg}")
    sys.exit(retcode)


def create_action(window_or_application, name, function):
    """ create ancd connect action """
    action = Gio.SimpleAction.new(name, None)
    action.connect("activate", function)
    window_or_application.add_action(action)



#!/usr/bin/python3

# dialog_widget.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" dialog_widget.py contains classes to display dialogs """

import sys
import os
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.locale_utils import _, f
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Adw', '1')
    gi.require_version('Gtk', '4.0')
    from gi.repository import Adw, Gtk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f'{dir1}/dialog_widget.ui')
class DialogWidget(Gtk.Box):
    """ present dialogs """
    __gtype_name__ = 'DialogWidget'
    help_widget_dialog = Gtk.Template.Child()
    help_widget_label = Gtk.Template.Child()
    about_widget_dialog = Gtk.Template.Child()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        for key, val in self.abbreviations().items():
            setattr(self, key, val)

    def abbreviations(self):
        """
        this method should be overwritten, if DialogWidged class
        is used outside of cashbox.
        """
        return {"a": self.data(_("article")),
                "as_": self.data(_("articles")),
                "As": self.data(_("Articles")),
                "C": self.widget(_("Cashbox")),
                "c": self.data(_("count")),
                "cs": self.data(_("counts")),
                "n": self.data(_("name")),
                "ns": self.data(_("names")),
                "p": self.data(_("price")),
                "ps": self.data(_("prices")),
                "Pl": self.widget(_("Pricelist")),
                "R": self.widget(_("Receipt")),
                "S": self.widget(_("Sale"))}

    def widget(self, txt):
        """ text about widget will be blue """
        return f"<b><span foreground='blue'>{txt}</span></b>"

    def data(self, txt):
        """ text about data will be green """
        return f"<b><span foreground='green'>{txt}</span></b>"

    def help_dialog(self, win, title, txt):
        """ show help """
        dialog = self.help_widget_dialog
        label = self.help_widget_label
        dialog.set_title(title)
        label.set_label(txt)
        dialog.present(win)

    def about_dialog(self, win, txt, version="0.0"):
        """ show info about cashbox """
        about = self.about_widget_dialog
        about.set_comments(txt)
        about.set_developers(["Bernd Schumacher"])
        about.set_version(version)
        about.present(win)


if __name__ == '__main__':

    a1 = _("hallo")

    class DialogWindow(Adw.ApplicationWindow):
        """ Test Dialog Window """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            self.props.default_width = 350
            self.props.default_height = 600

            box = Gtk.Box()
            box.set_orientation(Gtk.Orientation.VERTICAL)
            self.set_content(box)

            button = Gtk.Button(label=_("test_help_dialog"))
            box.append(button)
            button.connect('clicked', self.on_test_help_dialog)

            button = Gtk.Button(label=_("test_about"))
            box.append(button)
            button.connect('clicked', self.on_test_about)

        def on_test_about(self, _button):
            """ show about widget """
            d = DialogWidget()
            win = self
            d.about_dialog(win, f(_("""\
one two three {d.As}""")))

        def on_test_help_dialog(self, _button):
            """ show dialog widget """
            d = DialogWidget()
            win = self
            d.help_dialog(win, _("test_title"), f(_("""\
one two three
{d.As} can be listed with {d.n} and {d.p} in {d.Pl}.""")))

    class MyApp(Adw.Application):
        """ test class App """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)
            self.connect('activate', self.on_activate)

        def on_activate(self, app):
            """ start test window """
            win = DialogWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# app.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" app.py features:
 * defines classes MinWindow and App
 * starts GTK4 main window of decision
 * starts tests of python files imported by decision
 * unifies the start of an applicaton
 * provides global variables glo.app, glo.version and glo.application_id
   * provides the current version of decision in App.version
   * allows to start decision multiple times with different app.application_id
   * allows to calculate win with App.app.get_active_window()
 * can reduce the window_size to simulate librem5 on desktop
 * allows caller to have less code
 * enables commandline usage provided by appargs.py
"""

import sys
import os
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.utils import reduce_window_size, create_action
    from cashbox.read_appargs import appargs
    from cashbox.read_css import read_css
    from cashbox.dialog_widget import DialogWidget
    from cashbox.locale_utils import _, f
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Adw, Gtk, Gio, GLib
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


class MinWindow(Adw.ApplicationWindow):
    """ class MinWindow usage:
    * without class MinWindow:
       0  class MyMainWindow(Adw.ApplicationWindow):
    * with class App:
       0  class MyMainWindow(MinWindow):
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if hasattr(appargs, "t1") and appargs.t1:
            reduce_window_size(self)


class App(Adw.Application):
    """ class App usage:
    * without class App:
       0  class MyApp(Adw.Application):
       1
       2      def __init__(self, *args, **kwargs):
       3          super().__init__(*args,
       4                           application_id="org.gtk.example",
       5                           **kwargs)
       6          self.connect('activate', self.on_activate)
       7
       8      def on_activate(self, app):
       9          win = MyMainWindow(application=app)
      10          win.present()
      11
      12  app = MyApp()
      13  app.run(sys.argv)
    * with class App:
       0  class MyApp(App):
       1
       2      def on_activate(self, app):
       3          self.win = MyMainWindow(application=app)
       4          self.win.present()
       5
       6  app = MyApp()
       7  app.run(sys.argv)
    * to provide appargs without starting a window
       0  App().run(sys.argv)
    * to calculate win:
       0  win = App.app.get_active_window()
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, application_id=appargs.application_id,
                         flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE |
                         Gio.ApplicationFlags.HANDLES_OPEN, **kwargs)

        application = self
        create_action(application, "about", self.about_action)

        self.connect('activate', self.on_activate)

        self.add_main_option("t1", 0, GLib.OptionFlags.NONE,
                             GLib.OptionArg.NONE, "test_small_display", None)

        self.add_main_option("currency", ord("c"), GLib.OptionFlags.NONE,
                             GLib.OptionArg.STRING, "Euro or Dollar", None)

        self.add_main_option("css-path", 0, GLib.OptionFlags.NONE,
                             GLib.OptionArg.STRING, "Path to css file", None)

        # -v is alredy used by doctests
        self.add_main_option("version", 0, GLib.OptionFlags.NONE,
                             GLib.OptionArg.NONE, "say version", None)

        self.set_option_context_parameter_string("FILE")
        self.set_option_context_summary("price list FILE")
        self.set_option_context_description("the price list FILE should "
                                            "contain lines with an article "
                                            "name followed by a price")

    def on_activate(self, app):
        """ this will normally be overwritten """

    def do_command_line(self, *args, **_kwargs):
        """ use gnome command line """
        # To make pylint happy "*args" was used instead of "command_line".
        # Because "*args" was used, instead of "command_line", to have the
        # same argument list as parent class, the next 2 lines are needed
        assert len(args) == 1
        command_line = args[0]

        opts = command_line.get_options_dict().end().unpack()
        args = command_line.get_arguments()

        appargs.read_appargs(opts, args[1:])
        read_css()

        if hasattr(appargs, "version") and appargs.version:
            print(appargs.application_version)
            sys.exit(0)

        self.activate()
        return 0

    def about_action(self, _action, _param):
        """ show cashbox's about dialog """
        d = DialogWidget()
        win = self.get_active_window()
        d.about_dialog(win, f(_("""\
{d.C} memorises the cost of {d.as_} and calculates the total price and \
change. It is intended for small clubs on a celebration, where members are \
not experienced in memorizing prices and doing mental arithmetic.

{d.C} consists of the 3 windows {d.Pl}, {d.S} and {d.R}, that can be switched \
as needed. {d.Pl} can add, delete or modify {d.as_}, {d.S} picks {d.as_} to \
sell and {d.R} shows the total price of the choosen {d.as_}.""")),
                       version=appargs.application_version)


if __name__ == '__main__':

    class MyMainWindow(MinWindow):
        """ test class MinWindow """

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

            box = Gtk.Box()
            self.set_content(box)

            hamburger = Gtk.MenuButton()
            hamburger.set_icon_name("open-menu-symbolic")
            box.append(hamburger)

            popover = Gtk.PopoverMenu()
            hamburger.set_popover(popover)

            menu = Gio.Menu.new()
            menu.append("Show About Dialog", "app.about")
            popover.set_menu_model(menu)

            button = Gtk.Button(label="click me")
            box.append(button)
            button.connect('clicked', self.clicked)

            print(f"main appargs.digits=<{appargs.digits}>")

            for i, j in appargs.items():
                print(f"<{i}>=<{j}>")

        def clicked(self, _button):
            """ print something to test button """
            print("clicked")

    class MyApp(App):
        """ test class App """

        def on_activate(self, app):
            """ start test window """
            win = MyMainWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/env python3

# cashbox.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" cashbox Application """

import sys
import os
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dirp = os.path.basename(dir1)
dir2 = os.path.dirname(dir1)
if dirp == "bin":
    sys.path.append(os.path.join(dir2, 'share/cashbox/python3'))
if dirp == "decision":
    sys.path.append(dir2)

try:
    from cashbox.article import Sale
    from cashbox.sale_widget import SaleWidget
    from cashbox.view_switch_window import ViewSwitchWindow
    from cashbox.receipt_widget import ReceiptWidget
    from cashbox.app import App
    from cashbox.pricelist_widget import PricelistWidget
    from cashbox.read_appargs import appargs
    from cashbox.locale_utils import _
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Gtk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


class MyApp(App):
    """ cashbox App """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.last_child_name = None

        # sale
        self.sale = Sale()

        self.pricelist_widget = None

    def on_activate(self, app):
        win = ViewSwitchWindow(application=app)

        # PricelistWidget
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin_top=12,
                      margin_end=12, margin_bottom=12, margin_start=12,
                      spacing=12)
        self.pricelist_widget = PricelistWidget(self.sale, win=win)
        box.append(self.pricelist_widget)
        win.stack.add_titled(box, "Pricelist", _("Pricelist"))

        if appargs.moreargs:
            self.pricelist_widget.read_files(appargs.moreargs)

        # SaleWidget
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin_top=12,
                      margin_end=12, margin_bottom=12, margin_start=12,
                      spacing=12)
        win.stack.add_titled(box, "Sale", _("Sale"))
        sale_widget = SaleWidget(self.sale, win=win)
        box.append(sale_widget)

        # ReceiptWidget
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin_top=12,
                      margin_end=12, margin_bottom=12, margin_start=12,
                      spacing=12)
        win.stack.add_titled(box, "Receipt", _("Receipt"))
        receipt_widget = ReceiptWidget(self.sale, win=win)
        box.append(receipt_widget)

        win.present()


myapp = MyApp()
myapp.run(sys.argv)



#!/usr/bin/python3

# article.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" define class Article """

import sys
import os
import re
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.read_appargs import appargs
    from cashbox.data_list import DataList
    from cashbox.locale_utils import _
    from cashbox.app import App
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import GObject
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


def cent2str(cents):
    """ return a string representing the given cent amount """
    return (f"{cents // appargs.cents}{appargs.separator}"
            f"{cents % appargs.cents:0{appargs.digits}.0f}")


def str2cent(txt, allow_sloppy=False):
    """ calculate cents from a given string """
    ret = 0
    if allow_sloppy:
        txt = re.sub(r"[.,]", appargs.separator, txt)
    try:
        price = float(txt)
    except ValueError:
        return ret
    number_of_digits = str(txt)[::-1].find(appargs.separator)
    if number_of_digits <= appargs.digits:
        ret = int(price * appargs.cents)
    return ret


class Article(GObject.Object):
    """ Atricle with name price and count """
    name = GObject.Property(type=str)
    price = GObject.Property(type=int)
    count = GObject.Property(type=int)

    def __init__(self, name, price, count=0):
        super().__init__()
        self.name = name
        self.price = price
        self.count = count

    def __str__(self):
        return f"({self.name},{cent2str(self.price)},{self.count})"

    def text(self):
        """ return text representing Article in Widgets """
        if self.count:
            ret = f"{self.name} {cent2str(self.price)} {self.count}"
        else:
            ret = f"{self.name} {cent2str(self.price)}"
        return ret


class Comment(GObject.Object):
    """ A Pricelist Comment """
    comment = GObject.Property(type=str)

    def __str__(self):
        return f"{self.comment}"

    def __init__(self, comment):
        super().__init__()
        self.comment = comment

    def text(self):
        """ return text representing Comment in Widgets """
        return str(self)


class DropDownHead(GObject.Object):
    """ DropDownHead is a header for a DropDown Widget.
    The Header itself can't be selected.
    """
    drop_down_head = GObject.Property(type=str)

    def __str__(self):
        return f"DropDownHead=<{self.drop_down_head}>"

    def __init__(self, drop_down_head):
        super().__init__()
        self.drop_down_head = drop_down_head

    def text(self):
        """ return text representing DropDownHead in Widgets """
        return str(self)


class Sale(DataList):
    """ Sale """

    def __init__(self):
        super().__init__()

        # self.main_list
        self.add_plus_list(pick=self.is_picked,
                           selfname="picked")  # self.picked
        self.add_plus_list(pick=self.is_unpicked,
                           selfname="unpicked")  # self.unpicked
        self.add_plus_list(pick=self.is_unpicked, sort=True,
                           item1=DropDownHead(_("select article")),
                           selfname="drop_unpicked")  # self.drop_unpicked

    def is_picked(self, item):
        """ Articles can be picked """
        ret = False
        if isinstance(item, Article):
            ret = item.count > 0
        return ret

    def is_unpicked(self, item):
        """ Articles may be unpicked """
        ret = False
        if isinstance(item, Article):
            ret = item.count == 0
        return ret

    def text(self):
        """
        This will make Sale ediable as text.
        All relevant information is included.
        """
        return "\n".join([data.text() for data in self.main_list])

    def get_article(self, name, picked=None):
        """
        get article with given name or None.
        picked:
          None: no matter if picked
          False: not picked
          True: picked
        """
        found = None
        for data in self.main_list:
            if isinstance(data, Article):
                if data.name == name:
                    if picked is True:
                        if data.count > 0:
                            found = data
                    elif picked is False:
                        if data.count == 0:
                            found = data
                    elif picked is None:
                        found = data
                    break
        return found

    def count_zero(self):
        """
        e.g. start with a new customer, without picked articles
        """
        for article in self.main_list:
            article.count = 0


if __name__ == '__main__':

    App().run(sys.argv)
    sale = Sale()

    for d in [Comment("one two three"), Article("Banana", 110, 1),
              Article("Apple", 200, 2), Article("Strawberry", 250, 3),
              Article("Pear", 335, 4), Article("Watermelon", 100, 5),
              Comment("four,five six"), Article("Blueberry", 200, 6)]:
        print(f"adding <{d}>")
        sale.main_list.append(d)
    assert f"{sale}" == "[one two three,(Banana,1.10,1),(Apple,2.00,2)," + \
                        "(Strawberry,2.50,3),(Pear,3.35,4)," + \
                        "(Watermelon,1.00,5),four,five six,(Blueberry,2.00,6)]"
    assert f"{sale.text()}" == """one two three
Banana 1.10 1
Apple 2.00 2
Strawberry 2.50 3
Pear 3.35 4
Watermelon 1.00 5
four,five six
Blueberry 2.00 6"""

    # next 2 lines will normally be done by a widget
    a = sale.get_article("Apple", picked=True)
    a.count = 0
    assert f"{sale.get_article('Apple', picked=True)}" == "None"
    assert f"{sale.get_article('Apple', picked=False)}" == "(Apple,2.00,0)"

    a.count = 3
    assert f"{sale.get_article('Apple', picked=False)}" == "None"
    assert f"{sale.get_article('Apple', picked=True)}" == "(Apple,2.00,3)"
    assert f"{sale.get_article('Apple')}" == "(Apple,2.00,3)"

    assert a.name == "Apple"
    assert a.price == 200
    assert a.count == 3

    print("all asserts have been ok")



#!/usr/bin/python3

# read_css.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" read_css.py reads .css file """

import sys
import os
import gi

sys.path.append(os.path.dirname(os.path.dirname(os.path.realpath(__file__))))

if __name__ == '__main__':
    from cashbox.app import App

try:
    from cashbox.read_appargs import appargs
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version('Adw', '1')
    from gi.repository import Adw, Gtk, Gdk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


def read_css():
    """ reads .css file """
    css_provider = Gtk.CssProvider()
    css_provider.load_from_path(appargs.css_path)

    styleContext = Gtk.StyleContext
    display = Gdk.Display.get_default()
    styleContext.add_provider_for_display(display, css_provider,
                                          Gtk.STYLE_PROVIDER_PRIORITY_USER)


if __name__ == '__main__':

    # test button
    #   should change color of label to red -
    #   defined in cashbox.css as "#invalid"
    HINT = """
Hint: to see color of label change, when pressing the button, run:

cat <<END >/tmp/read_css.css
#invalid {
  background: @error_color;
}
END
./read_css.py --css-path /tmp/read_css.css
"""

    print(HINT)

    class CssWindow(Adw.Window):
        """ test window using .css file """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            box = Gtk.Box()
            self.set_content(box)

            button = Gtk.Button(label="button")
            box.append(button)

            label = Gtk.Label(label="label")
            box.append(label)

            button.connect('clicked', self.clicked, label)

            for i, j in appargs.items():
                print(f"<{i}>=<{j}>")

        def clicked(self, _button, label):
            """ test calback for button """
            if label.get_name() == "invalid":
                label.set_name("normal")
            else:
                label.set_name("invalid")

    class MyApp(App):
        """ test class App """

        def on_activate(self, app):
            """ start test window """
            win = CssWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# pricelist_widget.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" pricelist widget """

import sys
import os
import doctest
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.utils import create_action
    from cashbox.article import Article, Comment, Sale, str2cent
    from cashbox.read_appargs import appargs
    from cashbox.cshbx import Cshbx
    from cashbox.dialog_widget import DialogWidget
    from cashbox.locale_utils import _, f
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

if __name__ == '__main__':
    from cashbox.app import App, MinWindow

try:
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Gtk, Gio
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f'{dir1}/pricelist_widget.ui')
class PricelistWidget(Gtk.Box):
    """ to edit a list of articles """
    __gtype_name__ = 'PricelistWidget'
    textview = Gtk.Template.Child()
    error = Gtk.Template.Child()
    file_dialog = Gtk.Template.Child()  # Gtk.FileDialog
    cashbox_file_filter = Gtk.Template.Child()
    red_tag = Gtk.Template.Child()
    green_tag = Gtk.Template.Child()
    orange_tag = Gtk.Template.Child()

    class LineInfo():
        "Info and Errors seen in one line"

        def __init__(self):
            self.n = None  # article name
            self.p = None  # article price
            self.c = None  # article count
            self.n_err = None  # seen n errors
            self.p_err = None  # seen p errors
            self.c_err = None  # seen c errors
            self.other_err = {"double_name": None,  # the name, that is double
                              "name_to_long": None}  # how many chars to long

        def line_ok(self):
            """ check if line is ok """
            return (self.p and self.n and self.c
                    and not self.other_err["double_name"]
                    and not self.other_err["name_to_long"])

        def check_name_len(self, name, max_len):
            """ check if name is to long """
            self.other_err["name_to_long"] = len(name) - max_len
            if self.other_err["name_to_long"] <= 0:
                self.other_err["name_to_long"] = None

    @Gtk.Template.Callback()
    def on_unmap_all(self, _widget):
        """ time to check list of articles """
        self.check_buffer()

    @Gtk.Template.Callback()
    def on_map_all(self, _widget):
        """ write sale-variable to widget-buffer """
        self.buffer.set_text(self.sale.text())

    def __init__(self, sale, win, **kwargs):
        super().__init__(**kwargs)

        self.win = win
        self.sale = sale
        self.buffer = self.textview.get_buffer()

        self.buffer.set_enable_undo(True)

        self.table = self.buffer.get_tag_table()
        self.cshbx = Cshbx()

        create_action(win, "undo", self.on_undo_action)
        create_action(win, "redo", self.on_redo_action)
        create_action(win, "save", self.on_save_dialog)
        create_action(win, "load", self.on_load_dialog)
        create_action(win, "append", self.on_append_dialog)
        create_action(win, "help_workflow", self.on_help_workflow)
        create_action(win, "help_syntax", self.on_help_syntax)

        self.table.add(self.red_tag)
        self.table.add(self.green_tag)
        self.table.add(self.orange_tag)

        self.buffer.connect('changed', self.on_buffer_changed)
        self.check_buffer()
        self.buffer.connect('notify::cursor-position', self.on_cursor)

        self.cursor_seen_in_line = (-1, -1)

        filters = Gio.ListStore.new(Gtk.FileFilter)
        filters.append(self.cashbox_file_filter)

        self.file_dialog.set_filters(filters)
        self.file_dialog.set_initial_folder(Gio.File.new_for_path(
                                            appargs.user_app_dir))

    def on_undo_action(self, _action, _param):
        """ undo last step """
        self.buffer.undo()

    def on_redo_action(self, _action, _param):
        """ redo last undo """
        self.buffer.redo()

    def on_cursor(self, buffer, pos):
        """ cursor has moved """
        pos = buffer.get_property("cursor-position")
        (a, b) = self.cursor_seen_in_line
        if pos < a or pos > b:
            self.check_buffer()

    def check_buffer_line(self, line):
        """ check for line for syntax errors """
        line_info = self.LineInfo()
        # get the last match of price in a line
        p_iter = list(self.cshbx.re_price.finditer(line))
        if p_iter:
            line_info.p = list(p_iter)[-1]
            a_line = line[:line_info.p.start(0)]
            line_info.n = self.cshbx.re_name.match(a_line)
            if not line_info.n:
                line_info.n_err = self.cshbx.re_some.match(a_line)

            c_line = line[line_info.p.end(0):]
            line_info.c = self.cshbx.re_count.match(c_line)
            if not line_info.c:
                line_info.c_err = self.cshbx.re_some.match(c_line)
        else:
            line_info.p_err = self.cshbx.re_some.match(line)
        return line_info

    def color_text(self, buffer, line_start, li):
        """
        Checks the line in <buffer> starting at <line_start>, that represent
        an article.
        Uses the regex results <n> (name), <p> (price) and <c> (optional count)
        and the corresponding errors if the regex failed.
        Returns a list of hints to fix the errors.

        green: ok
        orange: the marked text could be used if another part would be fixed
        red: the text itself must be fixed
        """
        def apply_tag(start, end, tag):
            iter_start = buffer.get_iter_at_offset(line_start + start)
            iter_end = buffer.get_iter_at_offset(line_start + end)
            buffer.apply_tag(tag, iter_start, iter_end)

        error = []
        if li.line_ok():
            apply_tag(li.n.start(2), li.n.end(2), self.green_tag)
            apply_tag(li.p.start(1), li.p.end(5), self.green_tag)
            apply_tag(li.p.end(5)+li.c.start(2), li.p.end(5)+li.c.end(2),
                      self.green_tag)
        else:
            if li.p:
                apply_tag(li.p.start(0), li.p.end(1), self.orange_tag)
                if li.n:
                    name_to_long = li.other_err["name_to_long"]
                    if name_to_long:
                        error += [f(_("name {name_to_long} chars to long"))]
                        apply_tag(li.n.start(2), li.n.end(2), self.red_tag)
                    elif li.other_err["double_name"]:
                        error += [_("name already exists")]
                        apply_tag(li.n.start(2), li.n.end(2), self.orange_tag)
                    else:
                        apply_tag(li.n.start(2), li.n.end(2), self.orange_tag)
                elif li.n_err:
                    apply_tag(li.n_err.start(2), li.n_err.end(2), self.red_tag)
                    error += [_("no space after article name")]
                else:
                    error += [_("no article name")]
                if li.c:
                    apply_tag(li.c.start(2)+li.p.end(4),
                              li.c.end(2)+li.p.end(4), self.orange_tag)
                elif li.c_err:
                    apply_tag(li.c_err.start(2)+li.p.end(5),
                              li.c_err.end(2)+li.p.end(5), self.red_tag)
                    error += [_("bad optional count")]
            elif li.p_err:
                apply_tag(li.p_err.start(0), li.p_err.end(0), self.red_tag)
                error += [_("no price found")]

        return error

    def on_buffer_changed(self, _buffer):
        """ time to check list of articles """
        self.check_buffer()

    def check_buffer(self):
        """
        check text buffer and return the current sale represented
        by the the buffer.

        if buffer2sale == True:
            write widget-buffer to sale
        """

        def get_text(start, end):
            return line[start:end]

        line = None

        self.sale.clear()

        # clear color tags
        (iter1, iter2) = self.buffer.get_bounds()
        self.buffer.remove_all_tags(iter1, iter2)
        text = self.buffer.get_text(iter1, iter2, False)

        hint = ""
        if text == "":
            hint = _("please add lines with article and price")
        else:
            names = []
            line_start = 0  # start of current line in text
            for (line_nr, line) in enumerate(text.splitlines()):

                cursor = self.buffer.get_property("cursor-position")

                if line_start <= cursor <= line_start+len(line):
                    self.cursor_seen_in_line = (line_start,
                                                line_start+len(line))

                if line.lstrip()[:1] == "#":
                    # comment starting with "#"
                    self.sale.main_list.append(Comment(line))
                else:
                    # n=article name, p=article price, c=article count
                    li = self.check_buffer_line(line)
                    if li.n:
                        name = get_text(li.n.start(2), li.n.end(2))
                        if name in names:
                            li.other_err["double_name"] = name
                        li.check_name_len(name, appargs.max_name_len)
                    error = self.color_text(self.buffer, line_start, li)

                    if li.line_ok():
                        names.append(name)
                        article = Article(
                            name, str2cent(get_text(li.p.start(2), li.p.end(2))
                                           + "." + get_text(li.p.start(4),
                                           li.p.end(4))),
                            int("0" + get_text(li.p.end(5) + li.c.start(2),
                                               li.p.end(5) + li.c.end(2))))
                        # eprint(f"article=<{article}>")
                        self.sale.main_list.append(article)
                    else:
                        # error: give error message, but handled like a comment
                        self.sale.main_list.append(Comment(line))

                    if line_start <= cursor <= line_start+len(line) and error:
                        hint = f"line {line_nr+1}: "+", ".join(error)

                line_start += len(line)+1

        self.error.set_label(hint)

    def on_save_dialog(self, _action, _param):
        """ save start """
        self.file_dialog.save(None, None, self.on_save_dialog_finish)

    def on_save_dialog_finish(self, dialog, task):
        """ save end """
        file = dialog.save_finish(task)

        (iter1, iter2) = self.buffer.get_bounds()
        text = self.buffer.get_text(iter1, iter2, False)

        path = file.get_path()
        if not path.endswith(appargs.conf_suffix):
            path = path+appargs.conf_suffix

        with open(path, 'w', encoding="utf-8") as p:
            p.write(text)

    def on_load_dialog(self, _action, _param):
        """ load start """
        self.file_dialog.open(None, None, self.on_load_dialog_finish)

    def on_load_dialog_finish(self, dialog, task):
        """ load start """
        file = dialog.open_finish(task)
        (iter1, iter2) = self.buffer.get_bounds()
        with open(file.get_path(), 'r', encoding="utf-8") as p:
            data = p.read()
            self.buffer.delete(iter1, iter2)
            self.buffer.insert(iter1, data)

    def on_append_dialog(self, _action, _param):
        """ append start """
        self.file_dialog.open(None, None, self.on_append_dialog_finish)

    def on_help_workflow(self, _action, _param):
        """ help """
        d = DialogWidget()
        win = self
        d.help_dialog(win, _("pricelist workflow"), f(_("""\
{d.As} are listed in a text with {d.n} and {d.p} and an optional {d.c} in \
{d.Pl}. \
This text can be directly edited. It can also contain blank lines and \
comments. \
The text can be saved, loaded and appended with with the hamburger file menu. \
The  hamburger menu has Undo and Redo buttons to return to a previous state.

It is possible to enter all {d.as_} before starting with {d.S}, or to enter \
each {d.a} when it is sold, or a mixture of both methods.

It may also be possible to make a photo of a table of prices, and to use ocr \
software to extract text from the photo, before running {d.C}. The text can \
then be loaded in {d.Pl}. Then {d.ns} and {d.ps} can be copied and pasted \
to the needed syntax.""")))

    def on_help_syntax(self, _action, _param):
        """ help syntax """
        d = DialogWidget()
        win = self
        d.help_dialog(win, _("pricelist syntax"), f(_("""\
{d.Pl} mainly consists of a large text field, where a line can represents an \
{d.a}. Lines can also be empty or contain a comment.

A line with an {d.a} contains a {d.n} and {d.p} and an optional {d.c} \
separated by spaces. Lines starting with '#' are comments.

The {d.p} consists of digits followed by '.' or ',' and 2 additional digits. \
Text before the {d.p} is the {d.n}. The {d.p} may be followed by digits \
that represents a {d.c} to show how often the {d.a} is currently sold.""")))

    def on_append_dialog_finish(self, dialog, task):
        """ append end """
        file = dialog.open_finish(task)
        (_iter1, iter2) = self.buffer.get_bounds()
        with open(file.get_path(), 'r', encoding="utf-8") as p:
            data = p.read()
            self.buffer.insert(iter2, '\n'+data)

    def read_files(self, files):
        """ read """
        if files:
            with open(files[0], 'r', encoding="utf-8") as p:
                data = p.read()
                self.buffer.set_text(data)


if __name__ == '__main__':

    doctest.testmod()

    class PricelistWindow(MinWindow):
        """ Test Class """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)
            sale = Sale()

            pricelist_widget = PricelistWidget(sale, win=self)
            self.set_content(pricelist_widget)

            # style_manager = app.get_style_manager()
            # style_manager.set_color_scheme(Adw.ColorScheme.PREFER_LIGHT)
            # style_manager.set_color_scheme(Adw.ColorScheme.PREFER_DARK)

            if appargs.moreargs:
                # read testdata from file
                pricelist_widget.read_files(appargs.moreargs)
            else:
                # create minimal testdata now
                article = [("Banana", 1.10, 1), ("Apple", 2.0, 0)]
                for a in article:
                    sale.main_list.append(Article(a[0], a[1], a[2]))

    class MyApp(App):
        """ test class App """

        def on_activate(self, app):
            win = PricelistWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# drop_down_widget.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" customized DropDown class """

import sys
import os
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.app import App, MinWindow
    from cashbox.article import Article, DropDownHead, Sale, cent2str
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Adw', '1')
    gi.require_version('Gtk', '4.0')
    from gi.repository import GObject, Gtk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f'{dir1}/drop_down_widget_row.ui')
class DropDownWidgetRow(Gtk.Box):
    """ row of DropDown """
    __gtype_name__ = 'DropDownWidgetRow'
    w_name = Gtk.Template.Child()
    w_price = Gtk.Template.Child()


@Gtk.Template(filename=f'{dir1}/drop_down_widget.ui')
class DropDownWidget(Gtk.Box):
    """ Widget which displays Rows """
    __gtype_name__ = 'DropDownWidget'
    drop_down = Gtk.Template.Child()

    def __init__(self, data_list, **kwargs):
        super().__init__(**kwargs)

        # Set model
        self.drop_down.set_model(data_list)

        # Set up the factory
        factory = Gtk.SignalListItemFactory()
        factory.connect("setup", self._on_factory_setup)
        factory.connect("bind", self._on_factory_bind)
        self.drop_down.set_factory(factory)

        data_list.bind_property(
            "n-items",
            self.drop_down, "sensitive",
            GObject.BindingFlags.SYNC_CREATE,
            self._calc_sensitive, None)

    def get_drop_down(self):
        """ return drop_down """
        return self.drop_down

    def _calc_sensitive(self, _a, b):
        ret = False
        if b > 1:
            ret = True
        return ret

    def _on_factory_setup(self, _factory, item):
        drop_down_widget_row = DropDownWidgetRow()
        item.set_child(drop_down_widget_row)

    def _on_factory_bind(self, _factory, list_item):
        box = list_item.get_child()
        w_name = box.get_first_child()
        w_price = w_name.get_next_sibling()
        data = list_item.get_item()
        if isinstance(data, Article):
            w_name.set_text(data.name)
            w_price.set_text(cent2str(data.price))
        else:
            assert isinstance(data, DropDownHead)
            w_name.set_text(data.drop_down_head)
            w_price.set_text("")


if __name__ == '__main__':

    class DropDownWindow(MinWindow):
        """ test Class """

        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            sale = Sale()

            article = [("Banana", 1.10, 0), ("Apple", 2.0, 0),
                       ("Strawberry", 2.50, 0), ("Pear", 3.35, 4),
                       ("Watermelon", 1.0, 5), ("Blueberry", 2.0, 6)]

            for f in article:
                sale.main_list.append(Article(f[0], f[1], f[2]))

            drop_down_widget = DropDownWidget(sale.drop_unpicked)
            drop_down = drop_down_widget.get_drop_down()
            drop_down.connect("notify::selected-item",
                              self.do_delete_selected_item)
            self.set_content(drop_down_widget)

        def do_delete_selected_item(self, dropdown, _b):
            """ test method """
            print("do_delete_selected_item")
            if dropdown.get_selected() > 0:
                article = dropdown.get_selected_item()
                print(f"do_delete_selected_item: article=<{article}>")
                dropdown.set_selected(0)
                article.count = 1

    class MyApp(App):
        """ test class App """

        def on_activate(self, app):
            win = DropDownWindow(application=app)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# view_switch_window.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

"""
switch between main windows in cashbox application
"""

import sys
import os
import gi
dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.app import App
    from cashbox.utils import reduce_window_size
    from cashbox.read_appargs import appargs
except ImportError as exc:
    print('Error: decision modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import Adw, Gtk
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


@Gtk.Template(filename=f"{dir1}/view_switch_window.ui")
class ViewSwitchWindow(Adw.ApplicationWindow):
    """ customized Adw.ViewStack """
    __gtype_name__ = 'ViewSwitchWindow'
    stack = Gtk.Template.Child()  # Adw.ViewStack

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        # because MinWindow is not known in view_switch_window.blp
        # ViewSwitchWindow(MinWindow) is not possible
        # and the next 2 lines have been copied from MinWindow
        if hasattr(appargs, "t1") and appargs.t1:
            reduce_window_size(self)


if __name__ == '__main__':
    # not possible:
    #   class MyViewSwitchWindow(ViewSwitchWindow):
    # because:
    #   TypeError: Inheritance from classes with @Gtk.Template decorators
    #   is not allowed at this time

    class MyApp(App):
        """ test class App """

        def on_visible_child_name(self, stack, _):
            """ test message if Window switches """
            print(f"on_visible_child_name: <{stack.get_visible_child_name()}>")

        def on_activate(self, app):
            """ start test window """
            win = ViewSwitchWindow(application=app)
            win.stack.connect('notify::visible-child-name',
                              self.on_visible_child_name)
            for i in [1, 2, 3]:
                box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL,
                              margin_top=12, margin_end=12, margin_bottom=12,
                              margin_start=12, spacing=12)
                win.stack.add_titled(box, f"name-{i}", f"Title {i}")

                lbl = Gtk.Label(label="Label {i}")
                box.append(lbl)
            win.present()

    myapp = MyApp()
    myapp.run(sys.argv)



#!/usr/bin/python3

# read_appargs.py
#
# Copyright:
#   Copyright (C) 2024-2025 Bernd Schumacher <bernd@bschu.de>
#
# License: GPL-3.0+
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#   .
#   This package is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   .
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <https://www.gnu.org/licenses/>.
# Comment:
#   On Debian systems, the complete text of the GNU General
#   Public License version 3 can be found in
#   "/usr/share/common-licenses/GPL-3".

""" read from gtk command line """

import sys
import os
import pathlib
from datetime import datetime
import gi

dir1 = os.path.dirname(os.path.realpath(__file__))
dir2 = os.path.dirname(dir1)
sys.path.append(dir2)

try:
    from cashbox.utils import err
except ImportError as exc:
    print('Error: cashbox modules not found.', exc)
    sys.exit(1)

try:
    gi.require_version('Gtk', '4.0')
    gi.require_version(namespace='Adw', version='1')
    from gi.repository import GLib, Adw, Gio
except (ImportError, ValueError) as exc:
    print('Error: Dependencies not met.', exc)
    sys.exit(1)


class AttrDict(dict):
    """ access dictionary as attributes """
    application_id = "de.bschu.cashbox"
    application_version = "0.3.3"
    max_name_len = 40
    conf_suffix = ".cshbx"

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError as e:
            # hasattr expects AttributeError instead of KeyError
            # (needed for doctest)
            classname = type(self).__name__
            msg = f'{repr(classname)} object has no attribute {repr(key)}'
            raise AttributeError(msg) from e

    def __setattr__(self, key, value):
        self[key] = value

    def __init__(self):
        """
        Set defaults for global variable "appargs".
        """
        # set default opts
        self.user_app_dir = os.path.join(GLib.get_user_data_dir(),
                                         self.application_id)
        self.system_app_dir = os.path.join("/", "usr", "share", "cashbox")
        self.css_path = os.path.join(self.system_app_dir, "cashbox.css")
        self.dark = None
        self.session = datetime.now().strftime('%Y-%m-%d')

        self.currency = "Dollar"
        self.test_small_display = False

    def read_appargs(self, opts, moreargs):
        """ get args from gnome """

        # set command_line opts
        for i in opts:
            self[i.translate({ord('-'): ord('_')})] = opts[i]

        # set command_line args
        self["moreargs"] = moreargs

        # do opts handling
        if self.currency == "Euro":
            self.update({"separator": ",", "symbol": "€", "digits": 2,
                         "cents": 10**2})
        elif self["currency"] == "Dollar":
            self.update({"separator": ".", "symbol": "$", "digits": 2,
                         "cents": 10**2})
        else:
            err(f"currency option <{appargs['currency']}> not Dollar or Euro")

        pathlib.Path(appargs.user_app_dir).mkdir(parents=True, exist_ok=True)


# init global variable appargs, to be included by other modules
if "appargs" not in globals():
    appargs = AttrDict()


if __name__ == '__main__':
    # to test try with option --help
    # to test try with option --number

    class App(Adw.Application):
        """ test class App """

        def __init__(self, *args, **kwargs):
            super().__init__(*args, application_id="de.bschu.cashbox",
                             flags=Gio.ApplicationFlags.HANDLES_COMMAND_LINE |
                             Gio.ApplicationFlags.HANDLES_OPEN, **kwargs)

            self.connect('activate', self.on_activate)

            self.add_main_option("number", ord("n"), GLib.OptionFlags.NONE,
                                 GLib.OptionArg.INT, "Test number", None)

            self.set_option_context_parameter_string("MOREARGS")
            self.set_option_context_summary("More Args")
            self.set_option_context_description("More Args to be used in "
                                                "appargs.moreargs")

        def do_command_line(self, *args, **_kwargs):
            # To make pylint happy "*args" was used instead of "command_line".
            # Because "*args" was used, instead of "command_line", to have the
            # same argument list as parent class, the next 2 lines are needed
            assert len(args) == 1
            command_line = args[0]
            opts = command_line.get_options_dict().end().unpack()
            args = command_line.get_arguments()

            appargs.read_appargs(opts, args[1:])

            self.activate()
            return 0

        def on_activate(self, _app):
            """ start test window """
            for var, val in appargs.items():
                print(f"<{var}>=<{val}>")
            assert appargs.currency == "Dollar"

    myapp = App()
    myapp.run(sys.argv)
